package LLI::RosterHandle;

# This provides some helper functions to our roster...
#
# $Id: RosterHandle.pm,v 1.1 2003/03/24 01:05:11 mankins Exp $

use strict;

use LLI::Presence;

my $VERSION = 1.00;

sub new 
  {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my ($self) = (@_);

    $self->{"table"} = $self->{"table"} || 'presence';

    bless ($self, $class);

    my $type = $self->{"type"} || ''; # default to full roster

    $self->refresh();  #sets PRESENCE with our current roster data

    return $self;
  }

sub add
  {
    my $self = shift;

    my ($presence) = @_;

    my $dbh = $self->{'dbh'};

    # Update in database
    my $statement = "INSERT INTO ".$self->{'table'}." SET jid = ?, type = ?";
    my $results = $dbh->do($statement, undef, $presence->jid(), $presence->type);

    # Update in memory
    $self->refresh();
  }

sub del
  {
    my $self = shift;

    my ($presence) = @_;

    my $dbh = $self->{'dbh'};

    # Update in database -- removes all entries with this JID!
    my $statement = "DELETE FROM ".$self->{'table'}." WHERE jid = ?";
    my $results = $dbh->do($statement, undef, $presence->jid());

    # Update in memory
    $self->{'JIDS'}->{$presence->jid()} = undef if $presence;
  }

sub refresh
  {
    # This sets up a cache of our database's roster
    # this may need to get re-worked as the numbers grow...

    my $self = shift;

    my ($type) = @_;

    my $dbh = $self->{'dbh'};

    my (@vars);

    my $statement = "SELECT * FROM ".$self->{'table'};
    if ($type)
      {
	# We know by the JID what type it is.  I guess this column could go away
	$statement .= " WHERE type = ?";
	push (@vars,$type);
      }
    my $result = $dbh->selectall_hashref($statement, "jid", undef, @vars );

    my (@jids);

    # if we have anything in our database, populate
    if ($result)
      {
        # Go through our results from the database and fill out our memory structures
        foreach my $jid (keys %{ $result })
          {
            # Create our LLI::Presence object with the data
            my $presence = new LLI::Presence;

            $presence->jid($result->{$jid}->{"jid"});
	    $presence->status($result->{$jid}->{"presence"}); # this status is for their online/offline

	    # populate subscription statuses
	    my %subscriptions;
	    $subscriptions{"internal"} = $result->{$jid}->{"subscriber"};
	    $subscriptions{"external"} = $result->{$jid}->{"subscription"};
	    $presence->subscription_status(\%subscriptions);

            push (@jids, $presence);

	    # Store this presence object in our Roster
	    $self->{'JIDS'}->{$jid} = $presence;
          }
      }

    $self->{'JID_LIST'} = \@jids;
  }

sub get_presence
  {
    # Returns the entire presence object
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid};
  }

sub get_type
  {
    # Returns the type of a JID thru presence object
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid}->type();
  }

sub subscription_status
  {
    # Returns the subscription status of a JID thru presence object
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid}->subscription_status();
  }

sub get_status
  {
    # Returns the type of a JID thru presence object
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid}->status();
  }

sub get_jids
  {
    # Returns all JIDs, or all matching a certain type
    my $self = shift;

    my ($type) = @_;

    if ($type)
      {
	my (@jids);

	foreach my $jid (@{ $self->{'JID_LIST'} })
	  {
	    if ($self->get_type($jid) eq $type)
	      {
		push (@jids, $jid); # a match, keep it.
	      }
 	  }

	return \@jids;
      }
    else
      {
	return $self->{'JID_LIST'};
      }
  }

sub ping
  {
    my $self = shift;

    my ($jid) = @_;

    return unless $jid;

    my $from = $self->{'jabber_from'};

    my $xml = '<presence type="probe" from="$from" to="$jid" />';

    # Do the send
    $self->{'jabber_connection'}->Send($xml);
  }

sub exists
  {
    my $self = shift;
    
    my ($jid) = @_;
    
    return $self->{'JIDS'}->{$jid} ? 1 : 0;
  }

sub set_subscription
  {
    # Ok, what does this mean?  
    # This means that it will set the database to show that we are of $status
    # type subscribed to the JID in question.
    #
    #    US ->  THEM
    #  
    #  So we're answering the question, what is the relationship between us and
    #  them...what type of subscription do we have of their presence?  
    #  Not to be confused with THEM subscribing to OUR presence.  No sir. clear as mud.

    my $self = shift;

    my ($jid, $type, $status) = @_;

    my $dbh = $self->{'dbh'};
    
    # if it's an internal/external subscription we're updating.
    my $field = ($type eq "external") ? "subscription" : "subscriber";

    # Update in database
    my $statement = "UPDATE ".$self->{'table'}." SET $field = ? WHERE jid = ?";
    my $results = $dbh->do($statement, undef, $status, $jid);

    return $results;
  }

sub request_subscription
  {
    my $self = shift;

    # This requests a foriegn entity to allow us to subscribe to their presence

    my ($jid) = @_;

    return unless $jid;

    my $dbh = $self->{'dbh'};

    my $from = $self->{'jabber_from'};

    if (!($self->exists($jid)))
      {
	# This jid isn't in our roster.  let's add it first
	$self->add($jid);
      }

    my $xml = '<presence to="'.$jid.'" type="subscribe" from="'.$from.'"></presence>';

    $self->{'jabber_connection'}->Send($xml);

    $self->set_subscription($jid, "external", 'requested');
  }

sub request_unsubscription
  {
    # This requests a foriegn entity to allow us to unsubscribe to their presence

    my $self = shift;

    my ($jid) = @_;

    return unless $jid;

    my $dbh = $self->{'dbh'};

    my $from = $self->{'jabber_from'};

    if (!($self->exists($jid)))
      {
	# This jid isn't in our roster.  we can't unsubscribe...
	return undef;
      }

    my $xml = '<presence to="'.$jid.'" type="unsubscribe" from="'.$from.'"></presence>';
    $self->{'jabber_connection'}->Send($xml);    

    $self->set_subscription($jid, "external", 'unrequested');
  }

sub requested_subscription
  {
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid}->requested_subscription($jid);
  }

sub is_subscriber
  {
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid}->is_subscriber($jid);
  }

sub is_external_subscription
  {
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid}->is_external_subscription($jid);
  }

sub is_internal_subscription
  {
    my $self = shift;

    my ($jid) = @_;

    return $self->{'JIDS'}->{$jid}->is_internal_subscription($jid);
  }

sub wants_our_presence
  {
    # I can't remember...
    my $self = shift;
    my ($jid) = @_;

    return $self->is_internal_subscription($jid);
  }

sub we_want_their_presence
  {
    # I can't remember...
    my $self = shift;
    my ($jid) = @_;
    
    return $self->is_external_subscription($jid);
  }

sub status
  {
    my $self = shift;
    my ($jid, $status) = @_;

    my $dbh = $self->{'dbh'};

    # Update in database
    my $statement = "UPDATE ".$self->{'table'}." SET presence = ? WHERE jid = ?";
    my $results = $dbh->do($statement, undef, $status, $jid);

    # Set in memory
    $self->{'JIDS'}->{$jid}->status($status);
  }

1;
